function Background() {

    var self = this;

    var loadXMLDoc = function(dname, callback) {

        try {

            xJSON.load(dname, callback, {
                type: "raw"
            });

        } catch (e) {

            console.log("Ooops... errror " + e);
            if (window.XMLHttpRequest) {
                xhttp = new XMLHttpRequest();
            } else {
                xhttp = new ActiveXObject("Microsoft.XMLHTTP");
            }
            xhttp.open("GET", dname);

            xhttp.onload = function() {
                callback(xhttp.response);
            }
            xhttp.onerror = function() {
                console.log("XMLHttpRequest error! " + xhttp.statusText);
                callback(null);
            }
            xhttp.onreadystatechange = function() {
                if (xhttp.readyState != 4) {
                    return;
                }
                if (xhttp.status != 200) {
                    console.log("XMLHttpRequest http status " + xhttp.status);
                    return;
                }
                var serverResponse = xhttp.responseText;
                console.log("XMLHttpRequest -> " + serverResponse);
            };
            xhttp.send();
        }
    }

    this.getCityObject = function() {

        var obj = store.get("selectedCity");
        console.log("getCityObject -> " + obj);
        if (obj) {
            obj.clang = this.getLang();
            return obj;
        }
        console.log("getCityObject -> null");

        if (0) return {
            aqi: "38",
            clang: "en",
            curl: "beijing",
            id: "Beijing",
            key: "Beijing",
            name: "Beijing (北京)",
            utime: 1405238400
        };
        return null;

    }

    var setIcon = function() {

        var obj = store.get("aqiData");
        if (obj != null) {
            var icon = "./img/aqicons/aqi." + obj.aqi + ".png";
            console.log("Setting icon " + icon);
            chrome.browserAction.setIcon({
                path: icon,
            });
        } else {
            console.log("Data is null in the local storage!");
        }
    }

    this.loadAQI = function(reload) {

        console.log("loadAQI (background)");
        var t = store.get("aqiTime");
        var dt = Math.abs(t - Date.now());
        if (!reload && dt < 5 * 60 * 1000) {

            console.log("Using cached data");
            setIcon();

        } else {
            var obj = this.getCityObject();

            if (obj != null) {
                var t = new Date().getTime();
                var url = "http://aqicn.org/aqicn/json/android/" + obj.key + "/json?" + t;
                console.log("Loading " + url + " (" + dt + ")");

                loadXMLDoc(url, function(data) {

                    if (data != null) {

                        var obj = null;
                        try {
                            obj = eval("(" + data + ")");
                        } catch (e) {}

                        if (obj != null) {
                            store.set("aqiData", obj);
                            store.set("aqiTime", Date.now());

                            console.log("Sending onAqiUpdate");
                            chrome.runtime.sendMessage({
                                method: "onAqiUpdate",
                                aqi: obj
                            });

                            chrome.tabs.getSelected(null,
                                function(tab) {
                                    console.log("tab.id=" + tab.id);
                                    chrome.tabs.sendMessage(tab.id, {
                                        method: "onAqiUpdate",
                                        aqi: obj
                                    });
                                }
                            );
                        }

                    } else {
                        console.log("Loading error!");
                    }
                    setIcon();


                });

            }
        }
    };

    this.saveOptions = function(ls) {

        var options = [];
        var opts = ["optcheckedbaidu", "optcheckedbing", "optcheckedgoogle", "lang"];

        for (opt in opts) {
            opt = opts[opt];
            var checked = ls[opt];
            if (typeof checked == "undefined") checked = "true";
            checked = (checked == "true") ? true : false;
            options.opt = checked;

            store.set(opt, checked);
        }
        return options;
    }

    this.getLang = function() {
        var lang = store.get("lang");
        if (typeof lang == "undefined") {
            lang = chrome.i18n.getUILanguage();
            if (lang == "zh-CN") lang = "cn";
            else if (lang == "zh-HK") lang = "hk";
            else if (lang == "zh-TW") lang = "hk";
            else if (lang == "ja") lang = "jp";
            else if (lang == "ko") lang = "kr";
            else lang = "en";
        }
        console.log("getLang() -> " + lang + " (" + chrome.i18n.getUILanguage() + ")");
        return lang;
    }

    this.setLang = function(lang) {
        store.set("lang", lang);
        return store.get("lang");
    }


    this.checkOption = function(opt) {

        var checked = store.get("optchecked" + opt);
        if (typeof checked == "boolean") {
            checked = checked ? "true" : "false";
        } else if (typeof checked == "undefined") {
            console.log("It is undefined!");
            checked = "true";
        }
        console.log(opt + " is " + (checked == "true" ? "checked" : "unchecked"));
        return checked == "true";

    }

    this.setPref = function(prefs) {
        var opts = ["baidu", "bing", "google"];

        for (opt in opts) {
            var id = "optchecked" + opts[opt];
            store.set(id, prefs[opts[opt]]);
        }

        store.set("lang", prefs.lang);

    }


    chrome.tabs.onUpdated.addListener(function() {
        this.reloadCount = 0;
        if (this.timeout == null) loader();
    });

    /* Reload every 20 minutes for 5 hours */
    this.reloadCount = 0;
    this.timeout = null;

    var loader = function() {
        self.loadAQI();
        console.log("Timeout Loader: n=" + self.reloadCount);
        if (self.reloadCount++ < 5 * 3) {
            this.timeout = setTimeout(function() {
                loader();
            }, 20 * 60 * 1000);

        } else {
            this.timeout = null;
        }
    }
    loader();


    return this;

}


chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
    console.log("onMessage: " + request.method);
    if (request.method == "getSelectedCity") {

        sendResponse(background.getCityObject());

    } else if (request.method == "setSelectedCity") {

        console.log("setSelectedCity: ");
        console.log(request.city);
        store.set("selectedCity", request.city);
        background.loadAQI(true);

    } else if (request.method == "loadAQI") {

        var aqi = store.get("aqiData");
        if (aqi != null) {
            aqi.clang = background.getLang();
            sendResponse(aqi);
        }
        background.loadAQI();

    } else if (request.method == "setLang") {

        sendResponse(background.setLang(request.lang));


    } else if (request.method == "saveOptions") {

        sendResponse(background.saveOptions(request.options));

    } else if (request.method == "checkOption") {

        sendResponse(background.checkOption(request.opt));

    } else {
        sendResponse({}); // snub them.
    }

});

var background = new Background();


function startBackground() {
    console.log("startBackground!");
    var obj = background.loadAQI();
    if (obj) chrome.runtime.sendMessage({
        method: "onAqiUpdate",
        aqi: obj
    });

}