function AqiUtils() {
    this.bgcolor = function(v) {
        if (v == "-") return '#aaaaaa';
        else if (v <= 50) return '#009966';
        else if (v <= 100) return '#ffde33';
        else if (v <= 150) return '#ff9933';
        else if (v <= 200) return '#cc0033';
        else if (v <= 300) return '#660099';
        else return '#7e0023';
    }

    this.fgcolor = function(v) {
        if (v == "-") return '#eeeeee';
        else if (v <= 50) return '#ffffff';
        else if (v <= 100) return '#000000';
        else if (v <= 150) return '#000000';
        else if (v <= 200) return '#ffffff';
        else if (v <= 300) return '#ffffff';
        else return '#ffffff';
    }

    this.getgraph = function(aqi, nhours) {

        var graph = "";
        try {
            if (typeof nhours == "undefined") nhours = 24;

            var pollutants = ["PM2.5", "PM10", "O3"];
            for (var p = 0; p < pollutants.length; p++) {
                var min = max = null;
                var pol = pollutants[p];
                for (var i = 0; i < nhours; i++) {
                    var idx = nhours - i - 1;
                    if (typeof aqi.historic[pol][idx] != "undefined") {
                        var v = aqi.historic[pol][idx];
                        if (min == null || min > v) min = v;
                        if (max == null || max < v) max = v;
                    }
                }
                var row = "";
                min = min * 9 / 10;
                min = 0;
                for (var i = 0; i < nhours; i++) {

                    var h = i + parseInt(aqi.uhour) - (nhours);
                    if (h < 24) h += 24;
                    if (h > 24) h -= 24;
                    if (h == 24) h = 0;

                    var hasBorder = (h == 6 - 1 || h == 12 - 1 || h == 24 - 1 || h == 18 - 1);
                    var style = hasBorder ? "border-right:1px solid #ccc;" : "";

                    var idx = nhours - i - 1;
                    if (typeof aqi.historic[pol][idx] != "undefined") {
                        var v = aqi.historic[pol][idx];
                        var c = this.bgcolor(v);
                        var h = Math.floor((v - min) * 10 / (max - min));
                        row += "<td style='" + style + "'><div style='height:" + h + "px;width:5px;background-color:" + c + ";'> </div>";
                    } else {
                        row += "<td style='" + style + "'>";
                    }
                }
                graph += "<tr><td>" + pol.toUpperCase() + ": </td>" + row + "</tr>";
            }
            var row = "";
            for (var i = 0; i < nhours; i++) {
                var h = i + parseInt(aqi.uhour) - (nhours);
                if (h < 24) h += 24;
                if (h > 24) h -= 24;
                if (h == 24) h = 0;

                if (h == 6 - 1 || h == 12 - 1 || h == 24 - 1 || h == 18 - 1) {
                    row += "<td colspan=2 style='border:0px solid black;text-align:center;font-size:8px'>" + (h + 1) + "</td>";
                    i += 1;

                } else {
                    row += "<td></td>";
                }

            }
            graph += "<tr><td></td>" + row + "</tr>";

            graph = "<table class='aqigraph' border=0>" + graph + "</table>";
        } catch (e) {
            console.log("Error " + e);
        }
        return graph;

    }

    this.getAqiBadge = function(aqi) {
        return '<span class="aqitag" style="background-color:' + aqiUtils.bgcolor(aqi.aqi) + '; color:' + aqiUtils.fgcolor(aqi.aqi) + '">' + aqi.aqi + "</span>";

    }
}

var aqiUtils = new AqiUtils();

function AqiLang() {


    this.getAqiText = function(aqi, l) {

        console.log("getAqiText(" + aqi + "," + l + ")");
        var aqiToLevel = function(aqi) {
            if (aqi == "-") return 0;
            if (aqi <= 50) return 1;
            if (aqi <= 100) return 2;
            if (aqi <= 150) return 3;
            if (aqi <= 200) return 4;
            if (aqi <= 300) return 5;
            return 6;
        }

        var values = chrome.i18n.getMessage("aqivalues" + l);
        var aqiLevelLabels = values.split(",");


        return aqiLevelLabels[aqiToLevel(aqi)].trim();
    }

    this.getText = function(t, l) {
        var texts = {
            air_quality: {
                en: " Air Quality",
                cn: "空气污染",
                hk: "空氣污染",
                jp: "大気汚染",
                kr: "대기 오염"
            }
        }
        if (typeof texts[t] != "undefined") {
            if (typeof texts[t][l] != "undefined")
                return texts[t][l];
            return texts[t].en;
        }
        return "???";
    }

    this.getTitle = function(aqi) {
        return aqi.name + this.getText("air_quality", aqi.clang)
    }
    return this;
}

var aqiLang = new AqiLang();


function AqiSettings() {

    var xself = this;
    this.cityKey = function(callback) {
        this.cityObject(function(o) {
            if (o) callback(o.key);
            else callback(null);
        });

    }

    this.cityObject = function(callback) {
        chrome.runtime.sendMessage({
            method: "getSelectedCity"
        }, function(city) {
            callback(city);
        });

    }

    this.setCityObject = function(city) {
        chrome.runtime.sendMessage({
            method: "setSelectedCity",
            city: city
        }, function(response) {});

    }

    var updateAQI = function(aqi) {

        if (typeof aqi.forecast[aqi.clang] != "undefined")
            aqi.weather = aqi.forecast[aqi.clang];
        else aqi.weather = aqi.forecast["en"];

        if (typeof aqi["info" + aqi.clang] != "undefined")
            aqi.info = aqi["info" + aqi.clang];

        if (aqi.clang == "hk" && typeof aqi["infocn"] != "undefined")
            aqi.info = aqi["infocn"];

        if (typeof aqi["utime" + aqi.clang] != "undefined")
            aqi.utime = aqi["utime" + aqi.clang];

        if (aqi.clang == "hk" && typeof aqi["utimecn"] != "undefined")
            aqi.utime = aqi["utimecn"];

        if (aqi.clang == "cn" || aqi.clang == "hk" || aqi.clang == "jp" || aqi.clang == "kr") {
            aqi.name = aqi.namena;
        } else {
            aqi.name = aqi.nameen;
        }
        return aqi;

    }

    this.loadAQI = function(callback) {
        console.log("common: loadAQI");
        chrome.runtime.sendMessage({
            method: "loadAQI"
        }, function(aqi) {

            aqi = updateAQI(aqi);
            console.log("common: onLoadAQI:" + JSON.stringify(aqi));
            callback(aqi);

        });

    }

    this.checkOption = function(opt, callback) {
        chrome.runtime.sendMessage({
            method: "checkOption",
            opt: opt
        }, function(r) {
            callback(r);
        });

    }

    this.setOnAqiUpdate = function(f) {

        onAqiUpdate = f;
        setTimeout(function() {
            console.log("common: setOnAqiUpdate");
            chrome.runtime.onMessage.addListener(onMessage);
            xself.loadAQI(f);
        }, 200);

    }

    var onMessage = function(request, sender, sendResponse) {
        console.log("onMessage: " + request.method);
        if (request.method == "onAqiUpdate") {

            onAqiUpdate(updateAQI(request.aqi));

        }
    }

    var onAqiUpdate = function(aqi) {}



    return this;
}