"use strict";

/* -------------------------------------------------------------------- */

var self = require("sdk/self");
var buttons = require('sdk/ui/button/action');
var tabs = require("sdk/tabs");

var toolbarButton = buttons.ActionButton({
    id: "aqicn-link",
    label: "Visit aqicn.org",
    icon: {
        "16": "./img/aqicn.16.png",
        "32": "./img/aqicn.32.png",
        "64": "./img/aqicn.64.png"
    },
    onClick: handleClick
});

function handleClick(state) {
    openPopupPanel();

    /* Send the start event */
    popupPanel.port.emit('onOpenPanel');
}

/* -------------------------------------------------------------------- */

const xulapp = require("sdk/system/xul-app");
const usingAustralis = xulapp.satisfiesVersion(">=29");
const {
    id: addonID
} = require("sdk/self");
const {
    getMostRecentBrowserWindow
} = require("sdk/window/utils");

var popupPanel = require("sdk/panel").Panel({
    contentURL: self.data.url("html/popup.html"),
    contentScriptFile: [
        self.data.url("js/jquery.js"),
        self.data.url("js/moment-with-langs.min"),
        self.data.url("js/chrome-porting.js"),
        self.data.url("js/background.js"),
        self.data.url("js/common.js"),
        self.data.url("js/popup.js"),
        self.data.url("js/citysearch.js"),
    ],
    contentScriptWhen: "ready",
    height: 510,
    width: 510,
    onHide: function() {
        if (usingAustralis) {
            toolbarButton.state("window", {
                checked: false
            });
        }
    }

});



function setReceivers(xobj, event) {

    xobj.port.on('loadJSON', function(obj) {
        console.log("loadJSON from main! " + obj.url);

        console.log(obj);
        loadJSON(obj, function(result) {

            xobj.port.emit('onJsonLoaded', {
                obj: obj,
                data: result.text
            });

        })
    });

    xobj.port.on('onSetIcon', function(obj) {

        console.log("setIcon: " + obj.path);
        toolbarButton.icon = obj.path;

    });

    xobj.port.on('onGetLocale', function(obj) {

        var i18n = self.data.load("_locales/en/messages.json");
        i18n = eval("(" + i18n + ")");
        xobj.port.emit('onGetLocale', i18n);

    });

    /* ----------------------------------- */

    var ss = require("sdk/simple-storage");
    var localStoreShadow = ss.storage.aqi;
    if (typeof localStoreShadow == "undefined") localStoreShadow = {};
    xobj.port.emit('syncstorage', JSON.stringify(localStoreShadow));

    xobj.port.on('syncstorage', function(obj) {

        var ss = require("sdk/simple-storage");
        var store = ss.storage.aqi;
        if (typeof store == "undefined") store = {
            utime: Date.now()
        };

        store[obj.name] = JSON.parse(obj.val);
        console.log("store: " + obj.name);
        ss.storage.aqi = store;
    });

    /* ----------------------------------- */

    var prefs = require("sdk/simple-prefs").prefs;
    xobj.port.emit('preferences', JSON.stringify(prefs));

    /* ----------------------------------- */

    xobj.port.emit('onReady', {});
}

setReceivers(popupPanel);

function openPopupPanel() {
    if (usingAustralis) {

        // taken from privacybadger
        // https://github.com/EFForg/privacybadgerfirefox/blob/master/lib/ui.js

        const buttonPrefix =
            'button--' + addonID.toLowerCase().replace(/[^a-z0-9-_]/g, '');

        const toWidgetID = function(id) {
            return buttonPrefix + '-' + id;
        }

        const nodeFor = function(obj) {
            return getMostRecentBrowserWindow().document.getElementById(toWidgetID(obj.id));
        }

        popupPanel.show({
            position: toolbarButton
        }, nodeFor(toolbarButton));

    } else {
        var mediator = Cc['@mozilla.org/appshell/window-mediator;1']
            .getService(Ci.nsIWindowMediator);
        var document = mediator.getMostRecentWindow('navigator:browser').document;
        var navBar = document.getElementById('ddg-toolbar-button');
        popupPanel.show({
            position: toolbarButton
        }, navBar);
    }

}

/* -------------------------------------------------------------------- */



/* -------------------------------------------------------------------- */

function loadJSON(obj, callback) {

    console.log("Loading (" + obj.url + ")");
    var Request = require("sdk/request").Request;
    var req = Request({
        url: obj.url,
        onComplete: function(response) {

            console.log("Loaded (" + obj.url + ")");
            callback(response);
        }
    });
    req.get();

}

/* -------------------------------------------------------------------- */

var pageMod = require("sdk/page-mod");

var onAttach = function startListening(worker) {

    setReceivers(worker);
    /*
    worker.port.on('loadJSON', function(obj) {
        console.log("loadJSON from main! " + obj.url);

        console.log(obj);
        loadJSON(obj, function(result) {

            worker.port.emit('onJsonLoaded', {
                obj: obj,
                data: result.text
            });

        })
    });
*/
};

pageMod.PageMod({
    include: /^https?:\/\/www\.baidu\.com\/$/,
    contentStyleFile: [
        self.data.url("css/common.css"),
        self.data.url("css/otions.css")
    ],
    contentScriptFile: [
        self.data.url("js/jquery.js"),
        self.data.url("js/chrome-porting.js"),
        self.data.url("js/background.js"),
        self.data.url("js/common.js"),
        self.data.url("js/baidu.js"),
    ],
    onAttach: onAttach
});

pageMod.PageMod({
    include: /^https?:\/\/www\.bing\.com\/$/,
    contentScriptWhen: 'ready',
    contentStyleFile: [
        self.data.url("css/bing.css"),
        self.data.url("css/common.css"),
        self.data.url("css/otions.css")
    ],
    contentScriptFile: [
        self.data.url("js/jquery.js"),
        self.data.url("js/chrome-porting.js"),
        self.data.url("js/background.js"),
        self.data.url("js/common.js"),
        self.data.url("js/bing.js"),
    ],
    onAttach: onAttach
});

pageMod.PageMod({
    include: /^https?:\/\/(www|encrypted)\.google\..*\/.*$/,
    contentScriptWhen: 'ready',
    contentStyleFile: [
        self.data.url("css/google.css"),
        self.data.url("css/common.css"),
        self.data.url("css/otions.css")
    ],
    contentScriptFile: [
        self.data.url("js/jquery.js"),
        self.data.url("js/chrome-porting.js"),
        self.data.url("js/background.js"),
        self.data.url("js/common.js"),
        self.data.url("js/google.js"),
    ],
    onAttach: onAttach
});

console.log("=====================================================");